import { Injectable } from '@angular/core';
import { Http } from '@angular/http';
import { Observable } from 'rxjs/Rx';
import { Setting } from '../setting';


@Injectable()
export class AppConfig {

    private config: Array<Setting> = [];
    private env: Object = null;

    constructor(private http: Http) { }

    public getPresentationConfigs() {
        return this.config.filter(x => x.key.toLowerCase().endsWith("config"));
    }

    public get(key: string) {
        let setting = this.config.filter(x => x.key.toLowerCase() == key.toLowerCase());

        return setting.length > 0 ? setting[0].value : null;
    }

    public getEnv(key: any) {
        return this.env[key];
    }

    public load() {
        return new Promise((resolve, reject) => {
            this.http.get('./app/config/env.json').map(res => res.json()).catch((error: any): any => {
                console.log('Configuration file "env.json" could not be read');
                resolve(true);
                return Observable.throw(error.json().error || 'Server error');
            }).subscribe((envResponse) => {
                this.env = envResponse;
                let request: any = null;

                switch (envResponse.env) {
                    case 'production': {
                        request = this.http.get('./app/config/config.' + envResponse.env + '.json');
                    } break;

                    case 'development': {
                        request = this.http.get('./app/config/config.' + envResponse.env + '.json');
                    } break;

                    case 'default': {
                        console.error('Environment file is not set or invalid');
                        resolve(true);
                    } break;
                }

                if (request) {
                    request
                        .map((res: any) => res.json())
                        .catch((error: any) => {
                            console.error('Error reading ' + envResponse.env + ' configuration file');
                            resolve(error);
                            return Observable.throw(error.json().error || 'Server error');
                        })
                        .subscribe((responseData: any) => {
                            this.config = [];
                            for (let setting of responseData) {
                                this.config.push(new Setting(setting.Name, setting.Value));
                            }
                            resolve(true);
                        });
                } else {
                    console.error('Env config file "env.json" is not valid');
                    resolve(true);
                }
            });

        });
    }
}